/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net_nil
 *  @{
 */

/**
 * @file
 */

#ifndef __NET_NIL_MESSAGES_H__
#define __NET_NIL_MESSAGES_H__

#include <async.h>
#include <errno.h>

#include <ipc/ipc.h>

#include "../messages.h"

#include "../structures/measured_strings.h"
#include "../structures/packet/packet.h"

#include "device.h"

typedef enum {
	/* ( device_id, driver_service, mtu ) */
	NET_NIL_DEVICE = NET_NIL_FIRST,
	/* ( device_id, state ) */
	NET_NIL_DEVICE_STATE,
	/* ( device_id ), packet_send */
	NET_NIL_RECEIVED,
	/* ( device_id ), packet_send */
	NET_NIL_SEND,
	/* ( device_id ) -> addr, prefix, content, suffix */
	NET_NIL_PACKET_SPACE,
	/* ( device_id ), measured_strings_return( hardware address ) */
	NET_NIL_ADDR,
	/* ( device_id ), measured_strings_return( broadcast address ) */
	NET_NIL_BROADCAST_ADDR,
} nil_messages;

#ifdef NIL_BUNDLE

#include "../nil/nil_wrappers.h"

/** Returns the protocol service message parameter.
 */
#define NIL_GET_PROTO( call )		( services_t ) IPC_GET_ARG2( * call )

#define NIL_GET_MTU( call )			( size_t ) IPC_GET_ARG2( * call )

#define nil_device_state_msg( nil_phone, device_id, state )	\
	nil_device_state_wrapper( device_id, state )

#define nil_received_msg( nil_phone, device_id, packet, target )	\
	( void ) nil_receive_wrapper( device_id, packet )

#else

#define nil_get_addr( nil_phone, device_id, address, data )	\
	generic_get_addr( nil_phone, NET_NIL_ADDR, device_id, address, data )

#define nil_get_broadcast_addr( nil_phone, device_id, address, data )	\
	generic_get_addr( nil_phone, NET_NIL_BROADCAST_ADDR, device_id, address, data )

#define nil_send_msg( nil_phone, device_id, packet, sender )	\
	generic_send_msg( nil_phone, NET_NIL_SEND, device_id, packet_get_id( packet ), sender )

#define nil_packet_size_req( nil_phone, device_id, addr_len, prefix, content, suffix )	\
	generic_packet_size_req( nil_phone, NET_NIL_PACKET_SPACE, device_id, addr_len, prefix, content, suffix )

#define nil_device_state_msg( nil_phone, device_id, state )	\
	generic_device_state_msg( nil_phone, NET_NIL_DEVICE_STATE, device_id, state )

#define nil_received_msg( nil_phone, device_id, packet, target )	\
	generic_received_msg( nil_phone, NET_NIL_RECEIVED, device_id, packet_get_id( packet ), target )

#define nil_device_req( nil_phone, device_id, mtu, netif_service )	\
	generic_device_req( nil_phone, NET_NIL_DEVICE, device_id, mtu, netif_service )

#endif

#endif

/** @}
 */
